#version 120

//this is needed for bitwise operations
#extension GL_EXT_gpu_shader4 : enable
#define MAX_ITER 8


//inputs from code
	uniform sampler2D fbotex0;
	uniform sampler2D fbotex1;
	uniform sampler2D fbotex2;
	uniform sampler2D fbotex3;
	uniform sampler2D fbotex4;
	uniform sampler2D def0;

	uniform int Time;
	uniform vec2 resolution;

//inputs from vertex shader
	varying vec2 vTexCoord;

	varying vec4 lightNormal;

	varying float lightPower;
	varying vec3 lightColor;

	varying vec3 shadCol;
	
	varying float bgPower;
	varying float bgXPos;


//global variables
	vec3 diffuse;
	float depth;
	float lightEmit;
	float shadow;
	float shadowBias = 0.0018;





float rand(vec2 co, float ratio)
{
	return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453) * ratio;
}



float doAmbientOcclusion(vec2 tcoord, vec2 uv, vec3 cnorm)
{
	vec3 diff = texture2D(fbotex2, vec2(tcoord + uv)).rgb;
	vec3 v = normalize(diff);
	float d = length(diff);
	return max(0.0, dot(cnorm, v)) * (1.0/(1.0+d));
}



vec3 unpackColor(float col)
{
	int cl = int(col * 256.0);
	int r = 0;
	int g = 0;
	int b = 0;
	
	if (cl > 31.0){
		b = 128;
		cl -= 32;
	}
	if (cl > 15.0){
		b += 64;
		cl -= 16;
	}

	if (cl > 7.0){
		g = 128;
		cl -= 8;
	}
	if (cl > 3.0){
		g += 64;
		cl -= 4;
	}
	
	if (cl > 1.0){
		r = 128;
		cl -= 2;
	}
	if (cl > 0.0){
		r += 64;
	}
	
	return vec3(r / 256.0, g / 256.0, b / 256.0);
}

	


vec3 stars(float xpos, float ypos, float time)
{
	float v,t = v = .0005;
	vec3 cen = gl_FragCoord.xyz;
	cen.xy = ((cen.xy/resolution.xy)-0.5)*resolution.xy;
	cen.x = cen.x + xpos;
	cen.y = cen.y + ypos;
	for (float s=.0; s<2.; s+=.05) {
		vec3 p=s*(cen+vec3(.6,.6,.6))*t+vec3(.1,.2,fract(s+floor(time*25.)*.01));
		for (int i=0; i<5; i++) p=abs(p)/dot(p,p)-.8;
		v+=dot(p,p)*t;
	}
	return vec3(v, v, v * 5.0);
}



vec3 water(vec2 surfacePosition, float time)
{
	vec2 p = surfacePosition*3.0- vec2(15.0);
	vec2 i = p;
	float c = 1.0;
	float inten = .05;

	for (int n = 0; n < MAX_ITER; n++){
		float t = time * (1.0 - (3.0 / float(n+1)));
		i = p + vec2(cos(t - i.x) + sin(t + i.y), sin(t - i.y) + cos(t + i.x));
		c += 1.0/length(vec2(p.x / (2.*sin(i.x+t)/inten),p.y / (cos(i.y+t)/inten)));
	}
	
	c /= float(MAX_ITER);
	c = 1.5-sqrt(pow(c, 1.5));
	c = c*c;
	
	return vec3(c*0.3, c*0.5, c*0.925);
}


vec3 plasma(vec2 surfacePosition, float time)
{
	vec2 p = surfacePosition*5.0;
	vec2 i = p;
	float c = 0.0;
	float inten = 1.0;

	for (int n = 0; n < MAX_ITER; n++) {
		float t = time * (1.0 - (1.0 / float(n+1)));
		i = p + vec2(cos(t - i.x) + sin(t + i.y), sin(t - i.y) + cos(t + i.x));
		c += 1.0/length(vec2(p.x / (sin(i.x+t)/inten),p.y / (cos(i.y+t)/inten)));
	}
	c /= float(MAX_ITER);
	
	return vec3(pow(c,1.1))*vec3(1.0, 1.97, 1.0);
}




float rand(vec2 co) {
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}
 
float stars(float slow_time, vec2 position) {
	
	float brightness = 0.;
	
	for (int i = 0; i < 100; i++) {
		
		float rand_dist = .01 + 1.2 * rand(vec2(i));
		float rand_pos = 7.*rand(vec2(i+1));
		
		vec2 l_pos = rand_dist * vec2(sin(slow_time + rand_pos), cos(slow_time + rand_pos));
		
		vec2 dist = abs(l_pos - position);
		
		float intensity = 1. - (pow(dist.x, .17) + pow(dist.y, .1));
		intensity = clamp(intensity, 0., 1.);
		
		brightness = brightness + pow(4. * intensity, 2.);
	}
		return brightness;
}
 
float shooting_star(float slow_time, vec2 position, float speed, float size, float show_every_n) {
	
	// speed up
	slow_time *= speed;
	
	// show only every n-th pass of the shooting star
	if (mod(slow_time, show_every_n) < 5.) {
		
		float x = fract(slow_time) * 2. - 1.;
		
		vec2 ss_pos = vec2(x, sin(slow_time) / 2.);
		
		float center_intensity = size / length(position - ss_pos);
		
		// tail
		float tail_intensity = 0.;
		for (float tailStep = 0.; tailStep < 5.; tailStep += 1.) {
		  slow_time -= .001 * speed;
		
		  x = fract(slow_time) * 2. - 1.;
		
		  ss_pos = vec2(x, sin(slow_time) / 2.);
		
		  tail_intensity += size / length(position - ss_pos) / (tailStep + 1.);
		}
		
		return .5 * (center_intensity + tail_intensity);
		
	} else
	
		return 0.;
}
/*
vec3 starfield( void ) {
	
	vec2 position = ( gl_FragCoord.xy / resolution.xy ) * 2. - vec2(1.);
	position.y *= resolution.y / resolution.x;
	
	float slow_time = time / 8.;
	
	
	gl_FragColor = vec4(
		  stars(slow_time, position) +
		  shooting_star(slow_time, position, 4., .002, 3.) + 
		  shooting_star(slow_time, position, 14., .004, 2.) +
		  shooting_star(slow_time, position, -6., .0015, 3.) + 
		  shooting_star(slow_time, position, -12., .005, 2.)
		);
}
*/


void main()
{
	//channel 0 is the RGB
		vec4 deferred = texture2D(fbotex0, vec2(vTexCoord));
		float alpha = deferred.a;
		diffuse = deferred.rgb;

//		gl_FragColor = vec4(diffuse, 1.0);
//		gl_FragColor = vec4(alpha, 0.0, 0.0, 1.0);
//		return;



	//chanel 1 is the depth, shadow, lightEmit
		deferred = texture2D(fbotex1, vec2(vTexCoord));
		depth = deferred.r;
		lightEmit = deferred.b;
		shadow = deferred.g;

		//do the bg fade
			if (depth < 0.1){
//				vec3 bgColor = vec3((1.0-vTexCoord.y) * bgPower, 0.0, 0.0);
//				vec3 bgColor = vec3(bgPower, 0.0, 0.0);
				vec3 bgWater = water(vec2(vTexCoord.x, vTexCoord.y-bgPower+10), bgXPos + (Time * 0.0001)) * (bgPower * 2);
//				vec3 bgStars = plasma(-vec2(vTexCoord.x, vTexCoord.y-bgPower), bgXPos + (Time * 0.00001));


				float xp = (bgXPos * 300);
				mod(xp, resolution.x);
				vec3 bgStars = stars(xp, -bgPower * 200.0, 0) * clamp(1.0-(bgPower * 2), 0, 1.0);


				
				vec3 bgColor = vec3((1.0-vTexCoord.y) * bgPower, 0.0, 0.0);
				bgColor = bgColor + bgStars + bgWater;
				gl_FragColor = vec4(bgColor, 1.0);
				return;
			}
//		gl_FragColor = vec4(depth, 0.0, 0.0, 1.0);
//		gl_FragColor = vec4(lightEmit, 0.0, 0.0, 1.0);
//		gl_FragColor = vec4(shadow, 0.0, 0.0, 1.0);
//		return;


//		gl_FragColor = vec4(diffuse*shadow, 1.0);
//		return;




		
	//extract the emitted light an apply it
		if (lightEmit > 0.0){
			diffuse *= (lightEmit+1.0);
			diffuse += (lightEmit * 0.6);
			diffuse = mix(diffuse*shadow, diffuse, lightEmit);
		}else{	
//			diffuse *= shadow * clamp(depth, 0.0, 1.0);
			shadow = clamp(1.0-shadow, 0.0, 1.0) * (lightPower * 0.5);

			diffuse = mix(diffuse, shadCol * diffuse, shadow);
			diffuse *= clamp(depth, 0.0, 1.0);
//			diffuse *= shadow;
		}
		
//		diffuse = vec3(shadow, 0.0, 0.0);
//		diffuse = vec3(depth, 0.0, 0.0);
//		diffuse = vec3(lightEmit, 0.0, 0.0);
//		diffuse = vec3(deferred.a, 0.0, 0.0);

//		gl_FragColor = vec4(diffuse, 1.0);
//		return;



	//channel 2 is the vertex normals
		deferred = texture2D(fbotex2, vec2(vTexCoord));
		vec4 vertexNormal = vec4(deferred.rgb, 1.0);

//		gl_FragColor = vertexNormal;
//		return;


	//channel 3 is the texture coords.xy, surface material
		deferred = texture2D(fbotex3, vec2(vTexCoord));
		float isFauxLight = 0.0;
		deferred.b = int(deferred.b * 100.0);
		if (deferred.b > 99.9){
			isFauxLight = 1.0;
			deferred.b -= 100.0;
		}	
		float materialShine = mod(deferred.b, 10) * 0.1; //0..1
		float materialReflect = int(deferred.b / 10.0) * 0.1; //0..1
		
//		diffuse = vec3(materialShine, 0.0, 0.0);
//		diffuse = vec3(materialReflect, 0.0, 0.0);

//		gl_FragColor = gl_FragColor = vec4(diffuse, 1.0);
//		return;
		


	//channel 4 is the light distance/
		vec4 channel4 = texture2D(fbotex4, vec2(vTexCoord));







	if (lightEmit == 0.0){
		//global light coming from the main light
//			diffuse *= lightMix;
//			diffuse *= lightColor * (channel4.r * lightPower);
//			diffuse = mix(diffuse * lightColor * (channel4.r * lightPower), diffuse, lightMix);

			//how much light is shining on the surface
			float NdotL = max(0.0, dot(vertexNormal, lightNormal));
			if (NdotL < 0.0) NdotL = 0.0;
			diffuse *= (0.5+(NdotL * 0.5)) * lightPower * lightColor;

		//how much light is reflected of the material 0 = no reflection, 1.0 is lots of reflected light
//			float NdotL = dot(vertexNormal, lightNormal);
//			if (NdotL < 1.0) {
				diffuse *= 1.0+(NdotL * materialReflect);
//			}	

		//how shiny is the material	0..1
			vec4 materialSpecular = vec4(materialReflect, materialReflect, materialReflect, 1);
			
			float plane = dot(vertexNormal.xyz, lightNormal.xyz) * materialReflect;// * (lightFalloff * 0.02);
			if (plane < 0.0){
				plane = 0.0;
			}	
	
			diffuse += plane;
//		diffuse = vec3(plane, 0.0, 0.0);

		//specular lighting
			vec4 specular;
			specular = materialSpecular * pow(max(0.0, dot(vertexNormal.xyz, lightNormal.xyz)), materialShine);
			diffuse += specular.xyz;




		//ssao v2
			vec3 n = texture2D(fbotex2, vec2(vTexCoord)).rgb;
			float rnd = rand(vec2(vTexCoord), 0.02 * depth);
	
			float ao = 0.0;
			
			ao += doAmbientOcclusion(vec2(vTexCoord), vec2(1.2, 0.0)*rnd, n);
			ao += doAmbientOcclusion(vec2(vTexCoord), vec2(-1.2, 0.0)*rnd, n);
			ao += doAmbientOcclusion(vec2(vTexCoord), vec2(0.0, 1.2)*rnd, n);
			ao += doAmbientOcclusion(vec2(vTexCoord), vec2(0.0, -1.2)*rnd, n);
			
			ao += doAmbientOcclusion(vec2(vTexCoord), vec2(-1.0, -1.0)*rnd, n);
			ao += doAmbientOcclusion(vec2(vTexCoord), vec2(1.0, -1.0)*rnd, n);
			ao += doAmbientOcclusion(vec2(vTexCoord), vec2(-1.0, 1.0)*rnd, n);
			ao += doAmbientOcclusion(vec2(vTexCoord), vec2(1.0, -1.0)*rnd, n);
			
			ao *= 0.05;
			diffuse += ao-0.07;
	}		

//		gl_FragColor = gl_FragColor = vec4(diffuse, 1.0);
//		return;




	//channel 5 is the faux lighting system
		vec4 channel5 = texture2D(def0, vec2(vTexCoord));
		diffuse *= (channel5.rgb + vec3(1.0,1.0,1.0));

/*		
		float lightMix = 0.0;
		if (channel5.r > depth){// && channel5.r < depth + (channel5.g * 0.5)) {
			lightMix = channel5.g;
			//this version multiplys existing
//			diffuse *= (vec3(1.0, 1.0, 1.0) + (unpackColor(channel5.b) * lightMix * 4));
			//this version adds to all
			diffuse += (unpackColor(channel5.b) * lightMix * 4.0);
		}
//		diffuse += deferred.xyz;
//		diffuse = deferred.rgb;

//		gl_FragColor = gl_FragColor = vec4(diffuse, 1.0);
//		return;
*/





	//to render for output
	gl_FragColor = vec4(diffuse, 1.0);
}




