#version 120

//this is needed for bitwise operations
#extension GL_EXT_gpu_shader4 : enable


//internal constants
	const float shadowBias = 0.00005;


//inputs from code
	uniform vec4 lightPos;
	uniform vec4 lightLookat;
	uniform vec4 lightColor;
	
	uniform float shadowRange;
	uniform float shadowFalloff;

	// this is the light shadow map
	uniform sampler2D ShadowMap;


//inputs from vertex shader
	varying vec4 vertexDepth;

	varying vec4 vertexPos;
	varying vec4 vertexRaw;
	varying vec4 vertexNormal;

	varying vec4 lightNormal;

	varying vec4 ShadowCoord;


	varying float isLight;
	varying float isGlass;
	varying float isFauxLight;

	varying float materialShine;





void main(void){
	vec2 uv = gl_TexCoord[0].xy;

	float alpha = gl_Color.a;
	if (isFauxLight > 0.0){
		alpha = 0.0;
	}	
	

	//light emit
		float lightEmit = 0.0;
		if (isLight > 0.0){
			lightEmit = alpha;
			alpha = 1.0;
		}
	
	//depth buffer	
		//35 = default
		float depth = (1.0/(vertexDepth.z/20.0));
	 	float shadow = 0.0;
		

	//shadows
			//0.0 = dark shadows, 0.2 = normal shadows, 0.4 = faded shadows
//			float shadowRange = 0.1;
			//1.0 = small shadow, 0.5 = standard shadow, 0.1 = long dark shadow
//			float shadowFalloff = 0.3;

			vec4 shadowXY = ShadowCoord / ShadowCoord.w;
			
			// Used to lower moire pattern and self-shadowing
			shadowXY.z -= shadowBias;
			
			float distanceFromLight = texture2D(ShadowMap, shadowXY.xy).z;
	
//			float dist = (length(shadowXY - vertexRaw)*0.05);
			if (distanceFromLight < shadowXY.z){ //do shadow
//				shadow = dist * shadowFalloff + shadowRange;
				shadow = 1.0; //ok
//				float dist = (length(vertexPos - shadowXY)*0.2);
//				shadow = 1.0-clamp(dist * 0.2, 0.0, 1.0);
			}

			if (shadowXY.x < 0.0 || shadowXY.y < 0.0 || shadowXY.x > 1.0 || shadowXY.y > 1.0) { //always in shadow
				shadow = 1.0;
			}
			shadow = 1.0-shadow;
				



	// distance from light		
		float distFromLight = length(lightPos - vertexRaw) * 0.05;

	
//	diffuse = texture2D(tex1, vec2(vTexCoord));

//	diffuse = vec4(vertexEyeSpace.xyz, 1.0); 

//	diffuse = vec4(normalEyeSpace.xyz, 1.0); 

//	diffuse = vec4(1.0, 0.0, 0.0, 0.0);





	//channel 0 is the RGB
	gl_FragData[0] = vec4(gl_Color.rgb, alpha);

	//channel 1 is the depth, shadow, lightEmit
	gl_FragData[1] = vec4(depth, shadow, lightEmit, 1.0);

	//channel 2 is the vertexNormal
	gl_FragData[2] = vec4(vertexNormal.rgb, 1.0);

	//channel 3 is the texture coords.xy, surface material
	gl_FragData[3] = vec4(uv.xy, materialShine, 1.0);

	//channel 4 is light power/
	gl_FragData[4] = vec4(1.0-(distFromLight *0.2), 1.0, 0.0, 1.0);

}